/*
 * zeroII_uart.c
 *
 *  Created on: May 30, 2022
 *      Author: sichk
 */

#include "stm32f1xx_hal.h"
#include "zeroII_i2c.h"
#include "float.h"

typedef enum
{
	ZEROII_GET_STATUS = 0x5A,
	ZEROII_SET_SYSTEM_Z0 = 0xF2,
	ZEROII_GET_SYSTEM_Z0 = 0xC4,
	ZEROII_SET_FQ_GET_RX = 0x6D,
	ZEROII_SET_FQ_GET_RXSWRRL = 0xA3,
	ZEROII_GET_RX_DATA = 0x7C,
	ZEROII_GET_RX_SWR_RL = 0x9A,
	ZEROII_GET_FW_VERSION = 0xE5,
} ZeroII_Commands_e;

typedef enum {
	ZEROII_STATUS_BUSY_USB = 1,
	ZEROII_STATUS_BUSY_SPI,
	ZEROII_STATUS_BUSY_I2C,
	ZEROII_STATUS_BUSY_UART,
	ZEROII_STATUS_IDLE,
	ZEROII_STATUS_READY,
	ZEROII_STATUS_ERROR,
} ZeroII_Status_e;

GPIO_TypeDef *zeroII_rst_GPIO_Port = 0;
uint16_t zeroII_rst_GPIO_Pin = 0;

#define ZEROII_SLAVE_ADDR       0x5B
#define ZEROII_ADDRESS          ((ZEROII_SLAVE_ADDR) << 1)
#define I2C_READ               	1
#define I2C_WRITE              	0

#define zeroII_RESET_ON()		HAL_GPIO_WritePin(zeroII_rst_GPIO_Port, zeroII_rst_GPIO_Pin, GPIO_PIN_RESET)
#define zeroII_RESET_OFF()		HAL_GPIO_WritePin(zeroII_rst_GPIO_Port, zeroII_rst_GPIO_Pin, GPIO_PIN_SET)

#define I2C_IFC_TX_BUF_SZ		20
#define I2C_IFC_RX_BUF_SZ		20

uint8_t i2c_tx_buffer[I2C_IFC_TX_BUF_SZ] = { 0 };
uint8_t i2c_rx_buffer[I2C_IFC_RX_BUF_SZ] = { 0 };


uint32_t zeroII_operation_timestamp = 0;
uint32_t zeroII_minimal_delay = 10;
static bool zeroII_idle = false;
static bool measure_result_type = false;
static uint32_t systemZ0 = 50000;
static ZEROII_DEV_INFO_t zeroII_dev_info = {0};
static ZEROII_RESULT_t zeroII_meas_result = {0};

I2C_HandleTypeDef *zero_hi2c;

bool zeroII_is_idle(void);

void zeroII_error_handler()
{
	while(1);
}

void zeroII_init(I2C_HandleTypeDef *hi2c, GPIO_TypeDef *RESET_GPIO_Port, uint16_t RESET_GPIO_Pin)
{
	if (hi2c == 0) zeroII_error_handler();

	switch ((int32_t)RESET_GPIO_Port) {
#ifdef GPIOA
	case (int32_t)GPIOA:
	    __HAL_RCC_GPIOA_CLK_ENABLE();
		break;
#endif
#ifdef GPIOB
	case (int32_t)GPIOB:
	    __HAL_RCC_GPIOB_CLK_ENABLE();
		break;
#endif
#ifdef GPIOC
	case (int32_t)GPIOC:
	    __HAL_RCC_GPIOC_CLK_ENABLE();
		break;
#endif
#ifdef GPIOD
	case (int32_t)GPIOD:
	    __HAL_RCC_GPIOD_CLK_ENABLE();
		break;
#endif
#ifdef GPIOE
	case (int32_t)GPIOE:
	    __HAL_RCC_GPIOE_CLK_ENABLE();
		break;
#endif
#ifdef GPIOF
	case (int32_t)GPIOF:
	    __HAL_RCC_GPIOF_CLK_ENABLE();
		break;
#endif
#ifdef GPIOG
	case (int32_t)GPIOG:
	    __HAL_RCC_GPIOG_CLK_ENABLE();
		break;
#endif
#ifdef GPIOH
	case (int32_t)GPIOH:
	    __HAL_RCC_GPIOH_CLK_ENABLE();
		break;
#endif
#ifdef GPIOI
	case (int32_t)GPIOI:
	    __HAL_RCC_GPIOI_CLK_ENABLE();
		break;
#endif
#ifdef GPIOJ
	case (int32_t)GPIOJ:
	    __HAL_RCC_GPIOJ_CLK_ENABLE();
		break;
#endif
#ifdef GPIOK
	case (int32_t)GPIOK:
	    __HAL_RCC_GPIOK_CLK_ENABLE();
		break;
#endif
#ifdef GPIOL
	case (int32_t)GPIOL:
	    __HAL_RCC_GPIOL_CLK_ENABLE();
		break;
#endif
#ifdef GPIOM
	case (int32_t)GPIOM:
	    __HAL_RCC_GPIOM_CLK_ENABLE();
		break;
#endif
#ifdef GPION
	case (int32_t)GPION:
	    __HAL_RCC_GPION_CLK_ENABLE();
		break;
#endif
	default:
		zeroII_error_handler();
	}

	zeroII_rst_GPIO_Port = RESET_GPIO_Port;
	zeroII_rst_GPIO_Pin = RESET_GPIO_Pin;

	GPIO_InitTypeDef GPIO_InitStruct = {0};

    GPIO_InitStruct.Pin = zeroII_rst_GPIO_Pin;
    GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_InitStruct.Pull = GPIO_PULLUP;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
    HAL_GPIO_Init(zeroII_rst_GPIO_Port, &GPIO_InitStruct);

    zeroII_RESET_ON();
    HAL_Delay(10);
    zeroII_RESET_OFF();
    HAL_Delay(190);

    zero_hi2c = hi2c;

	zeroII_idle = false;
	zeroII_minimal_delay = 10;
	zeroII_operation_timestamp = HAL_GetTick();
}

// Device info
const ZEROII_DEV_INFO_t* const zeroII_get_info(void)
{
	if (zeroII_idle || zeroII_is_idle())
	{
		zeroII_idle = false;

		i2c_tx_buffer[0] = ZEROII_GET_FW_VERSION;
		if (HAL_I2C_Master_Transmit(zero_hi2c, ZEROII_ADDRESS | I2C_WRITE, i2c_tx_buffer, 1, 100) != HAL_OK)
			return false;
		if (HAL_I2C_Master_Receive(zero_hi2c, ZEROII_ADDRESS | I2C_READ, i2c_rx_buffer, 7, 100) != HAL_OK)
			return false;

		zeroII_minimal_delay = 10;
		zeroII_operation_timestamp = HAL_GetTick();

		zeroII_dev_info.version_maj = i2c_rx_buffer[0];
		zeroII_dev_info.version_min = i2c_rx_buffer[1];
		zeroII_dev_info.version_rev = i2c_rx_buffer[2];
		zeroII_dev_info.zeroII_serial = *((uint32_t*) (i2c_rx_buffer + 3));

		return (const ZEROII_DEV_INFO_t* const)&zeroII_dev_info;
	}
	return false;
}

// SystemZ0
float zeroII_get_system_z(void)
{
	if (zeroII_idle || zeroII_is_idle())
	{
		zeroII_idle = false;

		i2c_tx_buffer[0] = ZEROII_GET_SYSTEM_Z0;
		if (HAL_I2C_Master_Transmit(zero_hi2c, ZEROII_ADDRESS | I2C_WRITE, i2c_tx_buffer, 1, 100) != HAL_OK)
			return false;
		if (HAL_I2C_Master_Receive(zero_hi2c, ZEROII_ADDRESS | I2C_READ, i2c_rx_buffer, 4, 100) != HAL_OK)
			return false;

		zeroII_minimal_delay = 10;
		zeroII_operation_timestamp = HAL_GetTick();

		systemZ0 = *((uint32_t*) (i2c_rx_buffer));
		return ((float)systemZ0) / 1000.0F;
	}
	return false;
}

bool zeroII_set_system_z(float Z0)
{
	if ((Z0 > 0) && (zeroII_idle || zeroII_is_idle()))
	{
		zeroII_idle = false;

		systemZ0 = (uint32_t)(Z0 * 1000.0);

		i2c_tx_buffer[0] = ZEROII_SET_SYSTEM_Z0;
		*((uint32_t*) (i2c_tx_buffer + 1)) = systemZ0;
		if (HAL_I2C_Master_Transmit(zero_hi2c, ZEROII_ADDRESS | I2C_WRITE, i2c_tx_buffer, 5, 100) != HAL_OK)
			return false;

		zeroII_minimal_delay = 10;
		zeroII_operation_timestamp = HAL_GetTick();

		return true;
	}
	return false;
}

// Measure @ Frequency
bool zeroII_measure_RX(uint32_t fq)
{
	if ((100000 <= fq) && (fq <= 1000000000) && (zeroII_idle || zeroII_is_idle()))
	{
		zeroII_idle = false;

		i2c_tx_buffer[0] = ZEROII_SET_FQ_GET_RX;
		*((uint32_t*) (i2c_tx_buffer + 1)) = fq;
		if (HAL_I2C_Master_Transmit(zero_hi2c, ZEROII_ADDRESS | I2C_WRITE, i2c_tx_buffer, 5, 100) != HAL_OK)
			return false;

		measure_result_type = false;
		zeroII_minimal_delay = 50;
		zeroII_operation_timestamp = HAL_GetTick();

		return true;
	}
	return false;
}

bool zeroII_measure_RX_SWR_RL(uint32_t fq)
{
	if ((100000 <= fq) && (fq <= 1000000000) && (zeroII_idle || zeroII_is_idle()))
	{
		zeroII_idle = false;

		i2c_tx_buffer[0] = ZEROII_SET_FQ_GET_RXSWRRL;
		*((uint32_t*) (i2c_tx_buffer + 1)) = fq;
		if (HAL_I2C_Master_Transmit(zero_hi2c, ZEROII_ADDRESS | I2C_WRITE, i2c_tx_buffer, 5, 100) != HAL_OK)
			return false;

		measure_result_type = true;
		zeroII_minimal_delay = 50;
		zeroII_operation_timestamp = HAL_GetTick();

		return true;
	}
	return false;
}

bool zeroII_remeasure_RX(void)
{
	if (zeroII_idle || zeroII_is_idle())
	{
		zeroII_idle = false;

		i2c_tx_buffer[0] = ZEROII_GET_RX_DATA;
		if (HAL_I2C_Master_Transmit(zero_hi2c, ZEROII_ADDRESS | I2C_WRITE, i2c_tx_buffer, 1, 100) != HAL_OK)
			return false;

		measure_result_type = false;
		zeroII_minimal_delay = 50;
		zeroII_operation_timestamp = HAL_GetTick();

		return true;
	}
	return false;
}

bool zeroII_remeasure_RX_SWR_RL(void)
{
	if (zeroII_idle || zeroII_is_idle())
	{
		zeroII_idle = false;

		i2c_tx_buffer[0] = ZEROII_GET_RX_SWR_RL;
		if (HAL_I2C_Master_Transmit(zero_hi2c, ZEROII_ADDRESS | I2C_WRITE, i2c_tx_buffer, 1, 100) != HAL_OK)
			return false;
		measure_result_type = true;
		zeroII_minimal_delay = 50;
		zeroII_operation_timestamp = HAL_GetTick();

		return true;
	}
	return false;
}

// Check status
const ZEROII_RESULT_t* const zeroII_is_ready(void)
{
	uint32_t now = HAL_GetTick();

	if ((now - zeroII_operation_timestamp) < zeroII_minimal_delay) return false;

	i2c_tx_buffer[0] = ZEROII_GET_STATUS;
	if (HAL_I2C_Master_Transmit(zero_hi2c, ZEROII_ADDRESS | I2C_WRITE, i2c_tx_buffer, 1, 100) != HAL_OK)
		return false;
	if (HAL_I2C_Master_Receive(zero_hi2c, ZEROII_ADDRESS | I2C_READ, i2c_rx_buffer, 1, 100) != HAL_OK)
		return false;

	if (i2c_rx_buffer[0] == ZEROII_STATUS_READY)
	{
		zeroII_minimal_delay = 10;
		zeroII_operation_timestamp = HAL_GetTick();

		if (measure_result_type)
		{
			if (HAL_I2C_Master_Receive(zero_hi2c, ZEROII_ADDRESS | I2C_READ, i2c_rx_buffer, 16, 100) != HAL_OK)
				return false;

			zeroII_meas_result.R = *((float*) (i2c_rx_buffer));
			zeroII_meas_result.X = *((float*) (i2c_rx_buffer + 4));
			zeroII_meas_result.SWR = *((float*) (i2c_rx_buffer + 8));
			zeroII_meas_result.RL = *((float*) (i2c_rx_buffer + 12));
			return (const ZEROII_RESULT_t* const)&zeroII_meas_result;
		}
		else
		{
			if (HAL_I2C_Master_Receive(zero_hi2c, ZEROII_ADDRESS | I2C_READ, i2c_rx_buffer, 8, 100) != HAL_OK)
				return false;

			zeroII_meas_result.R = *((float*) (i2c_rx_buffer));
			zeroII_meas_result.X = *((float*) (i2c_rx_buffer + 4));
			zeroII_meas_result.SWR = 0.0;
			zeroII_meas_result.RL = 0.0;
			return (const ZEROII_RESULT_t* const)&zeroII_meas_result;
		}
	}
	else
	{
		zeroII_minimal_delay = 10;
		zeroII_operation_timestamp = HAL_GetTick();
		return false;
	}
	return false;
}

const ZEROII_RESULT_t* const zeroII_wait_until_ready(uint32_t timeout)
{
	uint32_t now = HAL_GetTick();

	if ((now - zeroII_operation_timestamp) < zeroII_minimal_delay)
		HAL_Delay(zeroII_minimal_delay - (now - zeroII_operation_timestamp));

	zeroII_minimal_delay = 10;

	while (timeout >= zeroII_minimal_delay)
	{
		i2c_tx_buffer[0] = ZEROII_GET_STATUS;
		if (HAL_I2C_Master_Transmit(zero_hi2c, ZEROII_ADDRESS | I2C_WRITE, i2c_tx_buffer, 1, 100) != HAL_OK)
			return false;
		if (HAL_I2C_Master_Receive(zero_hi2c, ZEROII_ADDRESS | I2C_READ, i2c_rx_buffer, 1, 100) != HAL_OK)
			return false;

		if (i2c_rx_buffer[0] == ZEROII_STATUS_READY)
		{
			zeroII_minimal_delay = 10;
			zeroII_operation_timestamp = HAL_GetTick();

			if (measure_result_type)
			{
				if (HAL_I2C_Master_Receive(zero_hi2c, ZEROII_ADDRESS | I2C_READ, i2c_rx_buffer, 16, 100) != HAL_OK)
					return false;

				zeroII_meas_result.R = *((float*) (i2c_rx_buffer));
				zeroII_meas_result.X = *((float*) (i2c_rx_buffer + 4));
				zeroII_meas_result.SWR = *((float*) (i2c_rx_buffer + 8));
				zeroII_meas_result.RL = *((float*) (i2c_rx_buffer + 12));
				return (const ZEROII_RESULT_t* const)&zeroII_meas_result;
			}
			else
			{
				if (HAL_I2C_Master_Receive(zero_hi2c, ZEROII_ADDRESS | I2C_READ, i2c_rx_buffer, 8, 100) != HAL_OK)
					return false;

				zeroII_meas_result.R = *((float*) (i2c_rx_buffer));
				zeroII_meas_result.X = *((float*) (i2c_rx_buffer + 4));
				zeroII_meas_result.SWR = 0.0;
				zeroII_meas_result.RL = 0.0;
				return (const ZEROII_RESULT_t* const)&zeroII_meas_result;
			}
		}

		HAL_Delay(zeroII_minimal_delay);
		timeout -= zeroII_minimal_delay;
	}
	zeroII_minimal_delay = 10;
	zeroII_operation_timestamp = HAL_GetTick();
	return false;
}


bool zeroII_is_idle(void)
{
	uint32_t now = HAL_GetTick();

	if ((now - zeroII_operation_timestamp) < zeroII_minimal_delay)
		HAL_Delay(zeroII_minimal_delay - (now - zeroII_operation_timestamp)); //return false;

	i2c_tx_buffer[0] = ZEROII_GET_STATUS;
	if (HAL_I2C_Master_Transmit(zero_hi2c, ZEROII_ADDRESS | I2C_WRITE, i2c_tx_buffer, 1, 100) != HAL_OK)
		return false;
	if (HAL_I2C_Master_Receive(zero_hi2c, ZEROII_ADDRESS | I2C_READ, i2c_rx_buffer, 1, 100) != HAL_OK)
		return false;

	zeroII_minimal_delay = 10;
	zeroII_operation_timestamp = HAL_GetTick();

	if (i2c_rx_buffer[0] == ZEROII_STATUS_IDLE)
	{
		zeroII_idle = true;
		return true;
	}
	else
		return false;
}
